package com.lantone.qc.kernel.catalogue.firstpagerecord;

import com.alibaba.fastjson.JSONArray;
import com.google.common.collect.Lists;
import com.lantone.qc.kernel.catalogue.QCCatalogue;
import com.lantone.qc.kernel.util.CatalogueUtil;
import com.lantone.qc.pub.Content;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.model.doc.FirstPageRecordDoc;
import com.lantone.qc.pub.model.doc.MedicalRecordInfoDoc;
import com.lantone.qc.pub.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * @ClassName : FIRP0199
 * @Description : 新生儿出生体重未填写
 * @Author : 胡敬
 * @Date: 2020-03-14 16:30
 */
@Slf4j
@Component
public class FIRP0199 extends QCCatalogue {
    private List<String> icdCodeList = Lists.newArrayList("Z37.0", "Z37.2", "Z37.3", "Z37.5", "Z37.6");

    public void start(InputInfo inputInfo, OutputInfo outputInfo) {
        /**
         *判断“新生儿出生体重”是否填写（以下任一满足）
         * 1.查证【住院信息表】或【病案首页的（出院科室/出院科别）】是否为包含（产科/新生儿科）任一字段存在时，“新生儿出生体重”必填
         * 2.查证病历内是否存在包含【新生儿评分/分娩记录/剖宫产】文书时，“新生儿出生体重”必填。
         * 3.【病案首页】>>（年龄不足一周岁的）【年龄】为小于等于28天的，“新生儿出生体重”必填。
         * 4.【病案首页】主要诊断或其他诊断编码中包含有（Z37.0/Z37.2/Z37.3/Z37.5/Z37.6）任一编码为开头时，“新生儿出生体重”必填。
         *
         * */
        status.set("0");
        //科室
        String behDeptName = "";
        //获取病案首页
        FirstPageRecordDoc firstPageRecordDoc = inputInfo.getFirstPageRecordDoc();
        //获取住院信息
        MedicalRecordInfoDoc medicalRecordInfoDoc = inputInfo.getMedicalRecordInfoDoc();
        if (firstPageRecordDoc == null) {
            return;
        }
        Map<String, String> firstpageStructureMap = inputInfo.getFirstPageRecordDoc().getStructureMap();
        //新生儿出生体重
        String newbornWeight = firstpageStructureMap.get(Content.newbornWeight);
        //出院科别
        String outDept = firstpageStructureMap.get(Content.outDept);
        //年龄
        String age = firstpageStructureMap.get(Content.age);
        if (StringUtil.isNotBlank(newbornWeight)) {
            return;
        }
        if (StringUtil.isNotBlank(outDept) && (outDept.contains("产科") || outDept.contains("新生儿科"))) {
            status.set("-1");
            return;
        }

        if (age.contains("天")) {
            try {
                int day = Integer.parseInt(age.replace("天", ""));
                if (day <= 28) {
                    status.set("-1");
                    return;
                }
            } catch (Exception e) {
                log.error("FIRP0199 类型转换错误" + e.getMessage(), e);
            }
        }

        String hospitalDiagnosis = firstpageStructureMap.get("出院诊断");
        if (StringUtil.isNotEmpty(hospitalDiagnosis)) {
            List<Map<String, String>> diagnosisMapList = (List<Map<String, String>>) JSONArray.parse(hospitalDiagnosis);
            for (Map<String, String> diagnosisMap : diagnosisMapList) {
                String diagnosisIcd = diagnosisMap.get("诊断编码");
                if (StringUtil.isNotEmpty(diagnosisIcd)) {
                    for (String icdCode : icdCodeList) {
                        if (diagnosisIcd.startsWith(icdCode)) {
                            status.set("-1");
                            Map<String, Object> structureExtMap = inputInfo.getFirstPageRecordDoc().getStructureExtMap();
                            String xmlId = CatalogueUtil.getXmlId(structureExtMap, Content.newbornWeight);
                            id.set(xmlId);
                            return;
                        }
                    }
                }
            }
        }

        if (medicalRecordInfoDoc != null) {
            //住院科室名称
            behDeptName = medicalRecordInfoDoc.getStructureMap().get("beh_dept_name");
        }
        if (StringUtil.isNotBlank(behDeptName) && (behDeptName.contains("产科") || behDeptName.contains("新生儿科"))) {
            status.set("-1");
            return;
        }

    }
}