package com.lantone.qc.kernel.catalogue.leavehospital;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.lantone.qc.kernel.catalogue.QCCatalogue;
import com.lantone.qc.kernel.client.SimilarityServiceClient;
import com.lantone.qc.pub.Content;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.model.doc.BeHospitalizedDoc;
import com.lantone.qc.pub.model.doc.LeaveHospitalDoc;
import com.lantone.qc.pub.model.entity.Annotation;
import com.lantone.qc.pub.model.entity.Diag;
import com.lantone.qc.pub.model.label.DiagLabel;
import com.lantone.qc.pub.model.vo.SimilarityVo;
import com.lantone.qc.pub.util.ListUtil;
import com.lantone.qc.pub.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @ClassName : LEA03328
 * @Description : 入院记录有【外（会）阴裂伤】，出院小结出院诊断无外（会）阴裂伤
 * @Author : Wsy
 * @Date: 2022-04-13 13:54
 */
@Component
public class LEA03328 extends QCCatalogue {
    @Autowired
    SimilarityServiceClient similarityServiceClient;

    @Override
    public void start(InputInfo inputInfo, OutputInfo outputInfo) {
        /**
         * （入院记录和"出院小结/出院记录"）任一不存在则通过
         * 1.查证【入院记录】>>【初步诊断】或【补充诊断】中是否有诊断能匹配【阴道或外阴（会阴）裂伤-疾病列表】中的疾病名称，或诊断能匹配正则【pdzz--阴道或外阴（会阴）裂伤】（参见正则列表），有则继续，且把能匹配的诊断（疾病）记存为：【list_A】(可能为1个或多个疾病)
         * 2.查证【出院小结/出院记录】>>【出院诊断】中，是否有任一个疾病和【list_A】中的疾病相同或相似（算法模型计算两个疾病的相似度，大于某个相似阈值即可），否则提示。
         *
         * */
        status.set("0");
        List<String> diagListWords = Content.JBLB_HYLS;
        //获取入院记录
        BeHospitalizedDoc beHospitalizedDoc = inputInfo.getBeHospitalizedDoc();
        //获取出院小结
        LeaveHospitalDoc leaveHospitalDoc = inputInfo.getLeaveHospitalDoc();
        if (beHospitalizedDoc == null && leaveHospitalDoc == null) {
            return;
        }
        //匹配疾病正则
        String rex = Content.PDZZ_HYLS;
        //创建存储能匹配疾病的集合
        List<String> diagList = new ArrayList<>();
        //入院记录不为空，获取其诊断信息
        if (beHospitalizedDoc != null) {
            //初步诊断
            DiagLabel initialDiagLabel = beHospitalizedDoc.getInitialDiagLabel();
            //补充诊断
            DiagLabel suppleDiagLabel = beHospitalizedDoc.getSuppleDiagLabel();
            if (initialDiagLabel != null) {
                List<Diag> diagLists = initialDiagLabel.getDiags();
                if (ListUtil.isNotEmpty(diagLists)) {
                    diagLists.stream().filter(diag -> StringUtil.isNotEmpty(diag.getHospitalDiagName()) && (diagListWords.contains(diag.getHospitalDiagName()) || diag.getHospitalDiagName().matches(rex))).forEach(diag ->
                            diagList.add(diag.getHospitalDiagName()));
                }

            }
            if (suppleDiagLabel != null) {
                List<Diag> diagLists = suppleDiagLabel.getDiags();
                if (ListUtil.isNotEmpty(diagLists)) {
                    diagLists.stream().filter(diag -> StringUtil.isNotEmpty(diag.getHospitalDiagName()) && (diagListWords.contains(diag.getHospitalDiagName()) || diag.getHospitalDiagName().matches(rex))).forEach(diag ->
                            diagList.add(diag.getHospitalDiagName()));
                }
            }
        }
        //如果疾病集合为空则直接返回
        if (ListUtil.isEmpty(diagList)) {
            return;
        }
        List<String> diagCollectList = diagList.stream().distinct().collect(Collectors.toList());
        if (ListUtil.isNotEmpty(diagCollectList)) {
            status.set("-1");
            //获取入院诊断
            DiagLabel beHospitalizedLabel = leaveHospitalDoc.getBeHospitalizedLabel();
            if (beHospitalizedLabel == null) {
                return;
            }
            List<Diag> diags = beHospitalizedLabel.getDiags();
            if (ListUtil.isEmpty(diags)) {
                return;
            }
            for (Diag diag : diags) {
                String diagName = diag.getHospitalDiagName();
                for (String diagCollectName : diagCollectList) {
                    if (StringUtil.isNotEmpty(diagName) && StringUtil.isNotEmpty(diagCollectName)) {
                        if (getLikeRate(diagName, diagCollectName)) {
                            status.set("0");
                            return;
                        }
                    }
                }
            }
            return;
        }
    }

    private boolean getLikeRate(String text_1, String text_2) {
        JSONArray similarContent = new JSONArray();
        JSONObject detailContent = new JSONObject();
        detailContent.put("text_1", text_1);
        detailContent.put("text_2", text_2);
        similarContent.add(detailContent);
        //存储CRF完整所需结构数据
        SimilarityVo similarityVo = new SimilarityVo();
        similarityVo.setData(similarContent);
        //获取CRF模型返回数据
        JSONArray data = getAnnotation(similarityServiceClient, similarityVo).getData();
        double likeRate = 0d;
        if (data.size() > 0) {
            JSONObject dataContent = data.getJSONObject(0);
            likeRate = dataContent.getDoubleValue("like_rate");
        }
        if (likeRate > 0.9) {
            return true;
        }
        return false;
    }

    private Annotation getAnnotation(SimilarityServiceClient similarityServiceClient, SimilarityVo similarityVo) {
        Annotation annotation = new Annotation();
        try {
            String annotation_str = similarityServiceClient.getAnnotation(similarityVo);
            annotation = JSON.parseObject(annotation_str, Annotation.class);
        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            return annotation;
        }
    }
}
